# smart text-to-html converter

from __future__ import generators
import types

def render(txt):
    "convert text to html in a non-broken way"
    lines = txt.split('\n')
    def line_iter():
        for line in lines:
            yield line + "\n"
    nextline = line_iter()
    stack = []
    output = []
    def add(x):
        if type(x) == types.ListType:
            output.extend(x)
        else:
            output.append(x)
    done = 0
    last_was_blank = 0
    while not done:
        try:
            line = nextline.next()
            if line.strip():
                last_was_blank = 0
            else:
                if last_was_blank:
                    continue
                last_was_blank = 1
        except StopIteration:
            done = 1
            line = ''
        def format(line):
            return line.replace('&&', '&amp;').replace('<<', '&lt;').replace('>>', '&gt;')
        def inblock():
            "returns (block_doesnt_need_p_tags, out_of_block_now)"
            firsttag = ''
            for x in stack:
                pos = x.find('>')
                if pos != -1:
                    firsttag += x[:pos+1]
                    break
                firsttag += x
            firsttag = firsttag.lstrip().lower()
            p = 0
            for blktag in ('ul', 'ol', 'code', 'pre'):
                if firsttag.startswith("<" + blktag):
                    return (1, ("".join(stack).find('</' + blktag) != -1))
                    break
            return (0, 0)
        def print_para():
            #print inblock()
            p = not inblock()[0]
            if p: add("<p>")
            sep = p and ' ' or ''
            add(sep.join([format(x) for x in stack]))
            if p: add("</p>")
            add("\n\n")
            stack[:] = []
        isblock, blockdone = inblock()
        #print "is",isblock, "done",blockdone, "".join(stack)
        if (blockdone or not isblock) and not line.strip():
            # blank line
            if len(stack):
                #print "print para",blockdone,isblock,not line.strip()
                print_para()
        else:
            if not line.strip():
                stack.append('\n')
            else:
                stack.append(line)
    return "".join(output)

if __name__ == '__main__':
    print render("""

this is a test.  i'm trying to make something that will make
<i>proper</i> html out of my text.  if I put two blank lines, it
should put <<p>> tags around it.  if I put double-<< chars (like this:
<<<<), it should turn them into html entities.

this is the second paragraph.

<i>this paragraph is in italics</i>

<ul>

<li>list item 1</li>

<li>list item 2</li>

</ul>


and now, an ordered list:

<ol><li>item 1</li><li>item 2</li>

<li>item 3</li>




<li>item 4</li></ol>

and a final paragraph.
broken up into two lines.  (should there be a <<br>> in there?)

now, something inside <<pre>> tags:

<pre>this is
   a
       test!</pre>

<code><pre>this is
some
code</pre></code>

    """)
